from django.core.exceptions import ValidationError

from post_office.models import EmailTemplate


def validate_template_name(template_name):
    try:
        EmailTemplate.objects.get(name=template_name)
    except EmailTemplate.DoesNotExist:
        raise ValidationError("No such EmailTemplate: %s" % template_name)


def validate_method_type(method_type):
    from unicef_notification.models import Notification

    if method_type not in (Notification.TYPE_CHOICES):
        raise ValidationError("Notification type must be 'Email'")



from django.conf import settings
from django.template import Origin, TemplateDoesNotExist
from django.template.loaders.base import Loader as BaseLoader

from post_office.utils import get_email_template

from unicef_notification.models import EmailTemplate

EMAIL_TEMPLATE_PREFIX = getattr(
    settings,
    "UNICEF_NOTIFICATION_EMAIL_TEMPLATE_PREFIX",
    'email-templates/'
)


class EmailTemplateLoader(BaseLoader):
    def get_template_sources(self, template_name):
        if not template_name.startswith(EMAIL_TEMPLATE_PREFIX):
            return

        yield Origin(
            name=template_name[len(EMAIL_TEMPLATE_PREFIX):],
            template_name=template_name,
            loader=self,
        )

    def load_template_source(self, template_name, template_dirs=None):
        for origin in self.get_template_sources(template_name):
            try:
                template = get_email_template(origin.name)
                return template.html_content, origin.name
            except EmailTemplate.DoesNotExist:
                pass
        raise TemplateDoesNotExist(template_name)

    def get_contents(self, origin):
        try:
            template = get_email_template(origin.name)
            return template.html_content
        except EmailTemplate.DoesNotExist:
            pass



import json

from django.conf import settings
from django.core import serializers
from django.db import models
from django.template import Context
from django.template.loader import get_template


def model_to_dictionary(obj):
    """
    Given a model instance `obj`, return a dictionary that represents it.
    E.g. something like
    {
        'pk': 15,
        'model':
        'audit.auditorstaffmember',
        'auditor_firm': 15,
        'user': 934
    }
    For _simple_ use from templates, this'll work as well as the model
    instance itself.
    And it's trivially serializable by the default json encoder.
    That's all we really need here.
    """
    # We cannot just use model_to_dict, because it excludes non-editable fields
    # unconditionally, and we want them all.

    # Note that Django's serializers only work on iterables of model instances

    json_string = serializers.serialize('json', [obj])
    # The string will deserialize to a list with one simple dictionary, like
    # {
    #     'pk': 15,
    #     'model':
    #     'audit.auditorstaffmember',
    #     'fields': {
    #         'auditor_firm': 15,
    #         'user': 934
    #     }
    # }
    d = json.loads(json_string)[0]
    # Promote the fields into the main dictionary
    d.update(**d.pop('fields'))
    return d


def serialize_dict(data):
    """
    Return a new dictionary, which is a copy of data, but
    if data is a dictionary with some model instances as values,
    the model instances are replaced with dictionaries so that
    the whole thing should be serializable.
    """
    return {
        k: model_to_dictionary(v) if isinstance(v, models.Model) else v
        for k, v in data.items()
    }


def strip_text(text):
    return '\r\n'.join([line.lstrip() for line in text.splitlines()])


def get_template_content(content, filename, context={}):
    # If content given, use that; if filename given, fetch the template
    # from that file and return its content; else, return an empty string.
    if content:
        return content
    if filename:
        ctx = Context(context)
        template = get_template(filename)
        return template.template.render(ctx)
    return ''


def send_notification(
        recipients,
        sender=None,
        from_address='',
        cc=None,
        context=None,
        subject=None,
        subject_filename=None,
        content=None,
        content_filename=None,
        html_content=None,
        html_content_filename=None,
):
    """
    Send a notification, building the content from templates and
    a rendering context.

    Always uses 'Email' type (no other notification type implemented yet).

    * recipients: list of email strings to address the notification to

    * cc: list of email strings to copy the notification to, or None

    * sender: any object. if present, stored as the ``.sender`` in the notification.
      If it's a User object, its ``.email`` is used as the notification
      "From" address.

    * from_address: If `sender` doesn't provide a From address, this can
      provide an email string to use for it. If this is None and the
      sender doesn't provide an address, then settings.DEFAULT_FROM_EMAIL
      is used.

    * context: dictionary used to render the templates, or None.

    Then, for each of subject, plain text message content, and html text message
    content, you can provide either the raw content, or the name of a template file.
    (If you provide both, the content will be used, not the template file).
    """
    from unicef_notification.models import Notification

    if not (sender or from_address):
        from_address = settings.DEFAULT_FROM_EMAIL

    # Let the model handle parameter validation by creating the instance
    # and 'cleaning' it before saving.
    subject = get_template_content(subject, subject_filename, context)
    text_message = get_template_content(content, content_filename, context)
    html_message = get_template_content(
        html_content,
        html_content_filename,
        context
    )

    if isinstance(recipients, str):
        recipients = [recipients]

    notification = Notification(
        method_type=Notification.TYPE_EMAIL,
        sender=sender,
        from_address=from_address,
        recipients=recipients,
        cc=cc or [],
        template_data=context,
        subject=subject,
        text_message=text_message,
        html_message=html_message,
    )
    notification.full_clean()
    notification.save()
    notification.send_notification()


def send_notification_with_template(
    recipients,
    template_name,
    context,
    sender=None,
    from_address='',
    cc=None,
):
    """
    Send an email notification using an EmailTemplate object as the source of
    the templates.

    Always uses 'Email' type (no other notification type implemented yet).

    * recipients: list of email strings to address the notification to

    * cc: list of email strings to copy the notification to, or None

    * sender: any object. if present, stored as the ``.sender`` in the notification.
      If it's a User object, its ``.email`` is used as the notification
      "From" address.

    * from_address: If `sender` doesn't provide a From address, this can
      provide an email string to use for it. If this is None and the
      sender doesn't provide an address, then settings.DEFAULT_FROM_EMAIL
      is used.

    * context: dictionary used to render the templates, or None.

    * template_name: name of email template to use (there must be a EmailTemplate
      record with that name)
    """
    from unicef_notification.models import Notification

    if not (sender or from_address):
        from_address = settings.DEFAULT_FROM_EMAIL

    if isinstance(recipients, str):
        recipients = [recipients]

    assert template_name

    # Let the model handle parameter validation by creating the instance
    # and 'cleaning' it before saving.
    notification = Notification(
        method_type=Notification.TYPE_EMAIL,
        sender=sender,
        from_address=from_address,
        recipients=recipients,
        cc=cc or [],
        template_name=template_name,
        template_data=context,
    )
    notification.full_clean()
    notification.save()
    notification.send_notification()



import logging
import os
from importlib import import_module

from django.apps import apps
from django.conf import settings
from django.core.management import BaseCommand
from django.db import transaction

from post_office.models import EmailTemplate

logger = logging.getLogger(__name__)

NOTIFICATION_TEMPLATE_DIR = getattr(
    settings,
    "UNICEF_NOTIFICATION_TEMPLATE_DIR",
    "notifications"
)


class Command(BaseCommand):
    help = 'Create Notifications command'

    @transaction.atomic
    def handle(self, *args, **options):
        logger.info('Command started')

        # loop through apps
        for app in apps.get_app_configs():
            # check if notification template dir exists
            notification_dir = "{}/{}".format(
                app.path,
                NOTIFICATION_TEMPLATE_DIR
            )
            if os.path.isdir(notification_dir):
                # walk through notification templates
                filenames = [
                    f for f in os.listdir(notification_dir)
                    if os.path.isfile(os.path.join(notification_dir, f))
                ]
                for filename in filenames:
                    if filename.startswith("__"):
                        continue
                    n = import_module("{}.{}.{}".format(
                        app.name,
                        NOTIFICATION_TEMPLATE_DIR,
                        filename.rsplit(".")[0]
                    ))
                    # creating email template objects
                    logger.info(n.name)
                    EmailTemplate.objects.update_or_create(
                        name=n.name,
                        defaults=n.defaults
                    )

        logger.info('Command finished')



# Generated by Django 2.0.6 on 2018-06-27 14:48

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models

import unicef_notification.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('post_office', '0006_attachment_mimetype'),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('method_type', models.CharField(default='Email', max_length=255, validators=[unicef_notification.validations.validate_method_type], verbose_name='Type')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True, verbose_name='Object ID')),
                ('from_address', models.CharField(blank=True, max_length=255, null=True)),
                ('recipients', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), blank=True, size=None, verbose_name='Recipients')),
                ('cc', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), blank=True, default=list, size=None)),
                ('sent_recipients', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), blank=True, default=list, size=None, verbose_name='Sent Recipients')),
                ('template_name', models.CharField(blank=True, default='', max_length=255, validators=[unicef_notification.validations.validate_template_name], verbose_name='Template Name')),
                ('template_data', models.JSONField(blank=True, null=True, verbose_name='Template Data')),
                ('subject', models.TextField(blank=True, default='')),
                ('text_message', models.TextField(blank=True, default='')),
                ('html_message', models.TextField(blank=True, default='')),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Content Type')),
                ('sent_email', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='post_office.Email')),
            ],
        ),
    ]



import json
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import ArrayField
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext as _

from model_utils import Choices
from post_office import mail
from post_office.models import EmailTemplate  # noqa used as a wrapper

from unicef_notification import validations
from unicef_notification.utils import serialize_dict

logger = logging.getLogger(__name__)


class Notification(models.Model):
    """
    Represents a notification instance from sender to recipients
    """

    TYPE_EMAIL = "Email"
    TYPE_CHOICES = Choices(
        (TYPE_EMAIL, 'Email'),
    )

    method_type = models.CharField(
        verbose_name=_('Type'),
        max_length=255,
        default=TYPE_EMAIL,
        validators=[validations.validate_method_type],
    )
    content_type = models.ForeignKey(
        ContentType,
        verbose_name=_('Content Type'),
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    object_id = models.PositiveIntegerField(
        verbose_name=_('Object ID'),
        null=True,
        blank=True,
    )
    sender = GenericForeignKey('content_type', 'object_id')
    # from_address can be used as the notification from address if sender is
    # not a user with an email address.
    from_address = models.CharField(max_length=255, null=True, blank=True)
    recipients = ArrayField(
        models.CharField(max_length=255),
        blank=True,
        verbose_name=_('Recipients')
    )
    cc = ArrayField(
        models.CharField(max_length=255),
        default=list,
        blank=True,
    )
    sent_recipients = ArrayField(
        models.CharField(max_length=255),
        default=list,
        blank=True,
        verbose_name=_('Sent Recipients')
    )
    # template_name has to be the name of an existing EmailTemplate object.
    # validate_template_name checks that.
    template_name = models.CharField(
        verbose_name=_('Template Name'),
        max_length=255,
        validators=[validations.validate_template_name],
        blank=True,
        default='',
    )
    # template_data is the context for rendering any templates.
    template_data = models.JSONField(
        verbose_name=_('Template Data'),
        null=True,
        blank=True,
    )
    # Save a link to the actual post_office.Email object that was sent
    sent_email = models.ForeignKey(
        'post_office.Email',
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    # Content of template used to render subject
    # if template_name not specified.
    subject = models.TextField(default='', blank=True)
    # Content of template used to render plain text message
    # if template_name not specified.
    text_message = models.TextField(default='', blank=True)
    # Content of template used to render HTML message
    # if template_name not specified.
    html_message = models.TextField(default='', blank=True)

    def __str__(self):
        return "{} Notification from {}: {}".format(
            self.method_type,
            self.sender,
            self.template_data
        )

    def __init__(self, *args, **kwargs):
        template_data = kwargs.pop('template_data', {})
        # Before trying to serialize template_data, we might need to
        # make it serializable
        try:
            json.dumps(template_data)
        except TypeError:
            assert isinstance(template_data, dict)
            template_data = serialize_dict(template_data)
        kwargs['template_data'] = template_data
        super(Notification, self).__init__(*args, **kwargs)

    def clean(self):
        if (self.text_message or self.html_message or self.subject) and self.template_name:
            raise ValidationError(
                "Notification cannot have both a template name, "
                "and a text_message or html_message or subject"
            )
        if not (self.text_message or self.html_message or self.template_name or self.subject):
            raise ValidationError(
                "Notification must have template name or text_message or html_message or subject."
            )
        # We won't require there to be any recipients, since callers might find
        # it easier to just call this with whatever list of email addresses
        # they have without having to first check whether they have any
        # addresses.
        # if not (len(self.cc) + len(self.recipients)):
        #     raise ValidationError("Notification must have at least one recipient or cc address.")

    def send_notification(self):
        """
        Dispatch notification based on type.
        """
        if self.method_type == self.TYPE_EMAIL:
            self.send_mail()
        else:
            # for future notification methods
            raise ValueError("Unknown notification type: %s" % self.method_type)

    def send_mail(self):
        User = get_user_model()

        if isinstance(self.sender, User):
            sender = self.sender.email
        elif self.from_address:
            sender = self.from_address
        else:
            sender = settings.DEFAULT_FROM_EMAIL

        if isinstance(self.template_data, str):
            template_data = json.loads(self.template_data)
        else:
            template_data = self.template_data

        try:
            email = mail.send(
                recipients=self.recipients,
                cc=self.cc,
                sender=sender,
                template=self.template_name,
                context=template_data,
                subject=self.subject,  # actually template text
                message=self.text_message,  # actually template text
                html_message=self.html_message,  # actually template text
            )
        except Exception:
            # log an exception, with traceback
            logger.exception('Failed to send mail.')
        else:
            self.sent_recipients = self.recipients + self.cc
            self.sent_email = email
            self.save()



#!/usr/bin/env python
import ast
import codecs
import os.path
import re
import sys
from codecs import open

from setuptools import find_packages, setup

ROOT = os.path.realpath(os.path.dirname(__file__))
init = os.path.join(ROOT, 'src', 'unicef_notification', '__init__.py')
_version_re = re.compile(r'__version__\s+=\s+(.*)')
_name_re = re.compile(r'NAME\s+=\s+(.*)')

sys.path.insert(0, os.path.join(ROOT, 'src'))

with open(init, 'rb') as f:
    content = f.read().decode('utf-8')
    VERSION = str(ast.literal_eval(_version_re.search(content).group(1)))
    NAME = str(ast.literal_eval(_name_re.search(content).group(1)))


setup(
    name=NAME,
    version=VERSION,
    url='https://github.com/unicef/unicef-notification',
    author='UNICEF',
    author_email='dev@unicef.org',
    license="Apache 2 License",
    description='Django package that handles sending of notifications',
    long_description=codecs.open('README.rst').read(),
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    include_package_data=True,
    install_requires=(
        'django',
        'django-model-utils',
        'django-post-office',
    ),
    extras_require={
        'test': (
            'flake8',
            'isort',
            'pytest',
            'pytest-cov',
            'pytest-django',
            'pytest-echo',
            'pytest-pythonpath',
            'factory-boy',
            'psycopg2-binary'
        )
    },
    platforms=['any'],
    classifiers=[
        'Environment :: Web Environment',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Intended Audience :: Developers',
    ],
    scripts=[],
)
